package caddyhugo

import (
	"fmt"
	"os/exec"
	"path"
	"time"

	"github.com/gohugoio/hugo/deps"
	"github.com/gohugoio/hugo/hugolib"
	"github.com/spf13/afero"

	"git.stephensearles.com/stephen/idleshut"
)

const (
	IdleWebsocketTimeout = 10 * time.Minute
	WebsocketFileTicker  = 1 * time.Second
)

// Publish really renders new content into the public directory
func (ch *CaddyHugo) Publish() error {
	err := ch.persistAllEdits()
	if err != nil {
		return err
	}
	cmd := exec.Command("hugo")
	cmd.Dir = ch.Dir
	_, err = cmd.CombinedOutput()
	if err != nil {
		return err
	}

	return nil
}

func buildSite(sites *hugolib.HugoSites) error {
	err := sites.Build(hugolib.BuildCfg{ResetState: true})
	if err != nil {
		return fmt.Errorf("caddy-hugo: building site: %v", err)
	}

	return err
}

func (ch *CaddyHugo) configWithFs(fs afero.Fs) (*hugolib.HugoSites, *deps.DepsCfg, error) {
	var err error
	cfg := &deps.DepsCfg{}
	cfgPath := path.Join(ch.Dir, "config.toml")
	cfg.Cfg, err = hugolib.LoadConfig(fs, "", cfgPath)
	if err != nil {
		return nil, cfg, fmt.Errorf("caddy-hugo: loading site configuration: %v", err)
	}
	cfg.Cfg.Set("workingDir", ch.Dir)

	sites, err := hugolib.NewHugoSites(*cfg)
	if err != nil {
		return nil, cfg, fmt.Errorf("caddy-hugo: initializing site: %v", err)
	}

	err = buildSite(sites)

	return sites, cfg, err
}

func HugoInternalProcessConfig(ch *CaddyHugo, es *editSession, touchFn func()) (idleshut.Config, error) {
	hugoSites, _, err := ch.configWithFs(es.tmpfs)
	if err != nil {
		return idleshut.Config{}, fmt.Errorf("caddy-hugo: loading site configuration: %v", err)
	}

	return idleshut.Config{
		TickDuration: WebsocketFileTicker,
		MaxIdleTicks: uint(IdleWebsocketTimeout/WebsocketFileTicker) + 1,
		Stop: func() error {
			ch.persistEditsForSession(es)

			es.doc.Close()

			ch.mtx.Lock()
			defer ch.mtx.Unlock()
			delete(ch.docs, es.filename)

			return nil
		},
		TickError: func(err error) {
			fmt.Println("error processing draft:", err)
		},
		Tick: func() error {
			err := afero.WriteFile(es.tmpfs, es.filename, []byte(es.doc.Contents()), 0644)
			if err != nil {
				return err
			}

			err = hugoSites.Build(hugolib.BuildCfg{ResetState: true})
			if err != nil {
				return err
			}

			return nil
		},
	}, nil
}
